/* Englisch <-> German translator
 * Based on Steak
 *
 * By Uwe Steinmann <Uwe.Steinmann@fernuni-hagen.de>
 *
 * Enjoy.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License  
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#else
#  define PACKAGE "gsteak"
#  define VERSION "0.2.0"
#endif
#include "build-defs.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>

#include <gnome.h>
#include <applet-widget.h>
//#include "mini_steak_icon.xpm"
#include "gsteak.h"


GtkWidget *applet;
GtkWidget *button;
GtkWidget *label;
GtkWidget *frame;
GtkWidget *vbox;
GtkWidget *counter;
GtkWidget *popup_window;
GtkWidget *process_clist;
GtkWidget *button_pixmap;
GdkPixmap *green_pixmap; 
GtkTooltips *tooltip;
int popup_width;
int popup_height;
int popup_open;

int kontext_variable=0;
int german_variable=0;
int english_variable=0;
int case_sensitive_variable=0;
int normal_and_or=0;  /* 1 steht fuer AND, 2 fuer OR und 0 fuer gar keine Verbindung */

#define LABEL_HEIGHT 13
#define APPLET_HEIGHT 49
#define APPLET_WIDTH 100


static void create_popup(GtkWidget *widget, gpointer data);
extern void prefs_cb(GtkWidget *w, gpointer p);

static void
about_cb(GtkWidget *w, gpointer d)
{
	gchar *authors[4];
	GtkWidget *about;
	
	authors[0] = "Uwe Steinmann <Uwe.Steinmann@fernuni-hagen.de>";
	authors[1] = "http://gehtnix.fernuni-hagen.de/gsteak";
	authors[2] = _("Insprired by xsteak");
	authors[3] = _("http://www.tm.informatik.uni-frankfurt.de/~razi/steak/steak.html");
	authors[4] = _("Expects steak programm at:");
	authors[5] = PROG_STEAK;
	authors[6] = NULL;
	
	about = gnome_about_new (
		"GNOME Translator",
		VERSION,
		"(C) 1999 Uwe Steinmann",
		(const gchar**)authors,
		_("This applet translates the word in the X window clipboard or any word entered."),
		NULL);
	
	gtk_widget_show(about);
	
	return;
}

static gint
appletDetached_signal(GtkWidget *widget, gpointer data) {
	/* applet has been detached; make it smaller */

	gtk_widget_set_usize(GTK_WIDGET(applet),
	  20,
	  APPLET_HEIGHT);

	return FALSE;
}

static gint
appletAttached_signal(GtkWidget *widget, gpointer data) {
	/* applet has been detached; restore original size */

	gtk_widget_set_usize(GTK_WIDGET(applet),
	  APPLET_WIDTH,
	  APPLET_HEIGHT);

	return FALSE;
}

static gint
commandKey_event(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	guint key = event->keyval;

	if(key == GDK_Return ||
	   key == GDK_KP_Enter ||
	   key == GDK_ISO_Enter ||
	   key == GDK_3270_Enter) {
		create_popup(widget, data);
		return 0;
	}
	return 1;
}

static gint close_pop(GtkWidget *widget,gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(process_clist));
	process_clist=NULL;

	gdk_pointer_ungrab(GDK_CURRENT_TIME);
	gtk_grab_remove(GTK_WIDGET(popup_window));
	gtk_widget_destroy(GTK_WIDGET(popup_window));
	widget=NULL;	
	
	popup_open=FALSE;	
	return FALSE;

}

#define MAXLINE 1024
#define DICTPROGRAM PROG_STEAK
void set_proc_list(int linenr)
{
	gchar *pcs[4];
	char *buf;
	FILE *fp;
	char *prog=DICTPROGRAM;
	char kkkk[3];
	char sprache[4];
	char nichtsandor[5];
	char command[MAXLINE];
	char line[MAXLINE];
	gint kontext_entries = 0;

	if (kontext_variable == 1) {
		strcpy(kkkk, "+k");
	} else {
		strcpy(kkkk, "-k");
	}

	if ((english_variable == 1) && (german_variable == 1)) {
		strcpy(sprache, "-de");
	} else if ((english_variable == 0) && (german_variable == 1)) {
		strcpy(sprache, "-d");
	} else {
		strcpy(sprache, "-e");
	}

	if (normal_and_or == 1) {
		strcpy(nichtsandor, "-and");
	} else if (normal_and_or == 2) {
		strcpy(nichtsandor, "-or");
	} else {
		strcpy(nichtsandor, sprache);
	}

	buf = gtk_entry_get_text(GTK_ENTRY(button));

	if (case_sensitive_variable == 1) {
		sprintf(command, "%s %s %s %s %s %s %s", prog, "-nowait", kkkk, sprache, "-noignor", nichtsandor, buf);
	} else {
		sprintf(command, "%s %s %s %s %s %s", prog, "-nowait", kkkk, sprache, nichtsandor, buf);
	}

	if ( (fp = popen(command, "r")) == NULL) {
		fprintf(stderr,"popen error");
		gtk_exit(0);
	}

	while ((fgets(line, MAXLINE, fp)) != NULL) {
		if ((strncmp("--------------------------------------------------",line,50) != 0) && (line[0] != '\n')) {
			if (strncmp("---------- Jetzt die Kontext-Eintraege ----------",line,49) == 0) {
				strcpy(line,"---------------- Jetzt die Kontext-Eintraege ----------------\n");
				kontext_entries = 1;
				pcs[0]="";
				pcs[1]=_("Context");
				pcs[2]="";
				gtk_clist_append(GTK_CLIST(process_clist),pcs);	
			} else if((strncmp("          English     ->     Deutsch",line,36) == 0) || (strncmp("          Deutsch     <->     English",line,37) == 0) || (strncmp("          Deutsch     ->     English",line,36) == 0)) {
			} else if((strncmp("Das Wort \"",line,10) == 0) || (strncmp("Kein Wort wurde mit dem Praefix \"",line,33) == 0 )) {
			} else if ((strncmp("Die Woerter mit Praefix \"",line,25) == 0)) {
			} else {
				char *ptr = line;
				while((*ptr != '\0') && (*ptr == ' '))
					ptr++;
				if(*ptr != '\0') {
					pcs[0] = ptr;
				while((*ptr != '\0') && (*ptr != ')'))
					ptr++;
				*ptr++ = '\0';
				while((*ptr != '\0') && (*ptr == ' '))
					ptr++;
				if(*ptr != '\0')
					pcs[1] = ptr;
				while((*ptr != '\0') && (*ptr != ':'))
					ptr++;
				*ptr++ = '\0';
				while((*ptr != '\0') && ((*ptr == ' ') || (*ptr == '=')))
					ptr++;
				if(*ptr != '\0')
					pcs[2] = ptr;
				gtk_clist_append(GTK_CLIST(process_clist),pcs);	 }
			}

		}
	}

	if (ferror(fp))
		printf("fgets error");
	pclose(fp);

//	pcs[0]="aldjf";
//	pcs[1]="lsjfd";
//	pcs[2]="Raw";
//	pcs[3]="Out";
}

static gint scroll_click(GtkWidget *widget, gpointer data)
{
  return TRUE;
}

GtkWidget* create_scroll_win(GtkWidget *widget, int linenr)
{
	GtkWidget *scrolled_window=NULL;
	char *nom_cols[3]={"Nr","Word","Translation"};
		
	scrolled_window=gtk_scrolled_window_new(NULL,NULL);
	
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
	     	                            GTK_POLICY_AUTOMATIC,
           	                        GTK_POLICY_AUTOMATIC);
    
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window),1);
 
	gtk_signal_connect(GTK_OBJECT(scrolled_window),"button_press_event"
                        ,GTK_SIGNAL_FUNC(scroll_click),NULL);

	process_clist=gtk_clist_new_with_titles(3,nom_cols);
	gtk_clist_column_titles_active(GTK_CLIST(process_clist));
		
	set_proc_list(linenr);
		
	gtk_clist_set_shadow_type (GTK_CLIST(process_clist), GTK_SHADOW_IN);
		
	gtk_clist_set_column_width(GTK_CLIST(process_clist),0,20);
	gtk_clist_set_column_width(GTK_CLIST(process_clist),1,146);
		 
	gtk_container_add(GTK_CONTAINER(scrolled_window),process_clist);
	gtk_widget_show(process_clist);

	gtk_widget_show(scrolled_window);
	return scrolled_window;
}

static gint update_proc_list(gpointer data)
{
	if(popup_open==TRUE)
	{
		gtk_clist_freeze(GTK_CLIST(process_clist));
		gtk_clist_clear(GTK_CLIST(process_clist));
		set_proc_list((int) data);
		gtk_clist_thaw(GTK_CLIST(process_clist));
	}
	return 1;
}

static void create_popup(GtkWidget *widget, gpointer data)
{
	gint scrX,scrY,posX,posY;	
	GtkWidget *frame_main;
	GtkWidget *scroll_win;
	GtkWidget *vbox;
	
	if(popup_open==FALSE)
	{
		popup_open=TRUE;
	
		popup_window=gtk_window_new(GTK_WINDOW_POPUP);

		gtk_window_set_policy(GTK_WINDOW(popup_window),0,0,1);
		gtk_window_set_title(GTK_WINDOW(popup_window),(gchar *)"Translation");
	
		gdk_window_get_pointer(NULL,&posX,&posY,NULL);

		scrX=gdk_screen_width();
		scrY=gdk_screen_height();
		if(posX+popup_width >= scrX)
			posX-=popup_width;
		if(posY-popup_height+10 <= 0)
			posY+=popup_height+20;

		gtk_widget_set_uposition(popup_window,posX,posY-popup_height-10);
		gtk_widget_set_usize(GTK_WIDGET(popup_window),popup_width,popup_height);
		gtk_widget_show(popup_window);

		scroll_win=create_scroll_win(popup_window, (int) data);

		gtk_signal_connect(GTK_OBJECT(popup_window),"button_press_event"
							,GTK_SIGNAL_FUNC(close_pop),NULL);


		vbox=gtk_vbox_new(FALSE,1);
     
		frame_main=gtk_frame_new(NULL);
		gtk_frame_set_shadow_type(GTK_FRAME(frame_main),GTK_SHADOW_OUT);
        
		
		gtk_box_pack_start(GTK_BOX(vbox),scroll_win,TRUE,TRUE,0);
		gtk_container_add(GTK_CONTAINER(frame_main),vbox);
		gtk_container_add(GTK_CONTAINER(popup_window),frame_main);
		gtk_widget_show(frame_main);
		
		gtk_widget_show(vbox);

		gtk_signal_connect(GTK_OBJECT(vbox),"button_press_event"
                        	,GTK_SIGNAL_FUNC(scroll_click),NULL);

	
		gdk_pointer_grab(popup_window->window,TRUE,
					     GDK_BUTTON_PRESS_MASK
            	        | GDK_BUTTON_RELEASE_MASK
                	    | GDK_ENTER_NOTIFY_MASK
                    	| GDK_LEAVE_NOTIFY_MASK
	                    | GDK_POINTER_MOTION_MASK,
    	                NULL,NULL,GDK_CURRENT_TIME);
                    
		gtk_grab_add(popup_window);

	}

}

static GtkWidget
*create_counter(void)
{
	GtkWidget *hbox, *handle, *frame2;
//	GdkBitmap *mask;
	GtkStyle *style;
	style = gtk_widget_get_style(applet);
/*
	green_pixmap = gdk_pixmap_create_from_xpm_d(applet->window, &mask,
	             &style->bg[GTK_STATE_NORMAL], (gchar **)mini_steak_icon_xpm);
*/
	tooltip=gtk_tooltips_new();
	gtk_tooltips_set_tip (tooltip, applet, "GNOME Translator", NULL);

	hbox = gtk_hbox_new(FALSE, 4);

	frame = gtk_frame_new(NULL);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 1);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

	vbox = gtk_vbox_new(FALSE, 1);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
/*	button = gtk_button_new();
	gtk_signal_connect(GTK_OBJECT(button),"clicked",
  		                   GTK_SIGNAL_FUNC(create_popup), (gpointer) NULL);
	button_pixmap = gtk_pixmap_new(green_pixmap, NULL);
	gtk_container_add(GTK_CONTAINER(button), button_pixmap);
*/
	button = gtk_entry_new();
	gtk_widget_set_usize(GTK_WIDGET(button), 80, -1);
	gtk_signal_connect(GTK_OBJECT(button), "key_press_event",
	  GTK_SIGNAL_FUNC(commandKey_event),
	  NULL);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
	label = gtk_button_new_with_label(_("Translate"));
	gtk_signal_connect(GTK_OBJECT(label),"button_press_event",
  		                   GTK_SIGNAL_FUNC(create_popup), (gpointer) NULL);
	gtk_widget_set_usize(label, -1, LABEL_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);

	handle = gtk_handle_box_new();
	gtk_signal_connect(GTK_OBJECT(handle), "child_detached",
	  GTK_SIGNAL_FUNC(appletDetached_signal),
	  NULL);
	gtk_signal_connect(GTK_OBJECT(handle), "child_attached",
	  GTK_SIGNAL_FUNC(appletAttached_signal),
	  NULL);
	gtk_container_add(GTK_CONTAINER(handle), hbox);
	gtk_widget_show(handle);
	frame2 = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame2), handle);
	return frame2;
}

static void
bye(GtkWidget *w, gpointer p)
{
	gnome_config_set_int("gsteak/general/width", popup_width);
	gnome_config_set_int("gsteak/general/height", popup_height);
	gnome_config_set_int("gsteak/general/german", german_variable);
	gnome_config_set_int("gsteak/general/english", english_variable);
	gnome_config_set_int("gsteak/general/context", kontext_variable);
	gnome_config_sync();
	gnome_config_drop_all();

	applet_widget_gtk_main_quit();
}

static int
save_session(GtkWidget *w, gpointer p)
{
	gnome_config_set_int("gsteak/general/width", popup_width);
	gnome_config_set_int("gsteak/general/height", popup_height);
	gnome_config_set_int("gsteak/general/german", german_variable);
	gnome_config_set_int("gsteak/general/english", english_variable);
	gnome_config_set_int("gsteak/general/context", kontext_variable);
	gnome_config_sync();
	gnome_config_drop_all();
	return FALSE;
}

int
main(int argc, char *argv[])
{
	/* initialize the i18n stuff */
	textdomain (PACKAGE);
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);

	applet_widget_init("gsteak_applet", VERSION, argc, argv, NULL, 0, NULL);
	applet = applet_widget_new("gsteak_applet");
	if(!applet)
		g_error("Cannot create gsteak applet!");

	popup_width = gnome_config_get_int("gsteak/general/width=410");
	popup_height = gnome_config_get_int("gsteak/general/height=270");
	kontext_variable = gnome_config_get_int("gsteak/general/context=1");
	english_variable = gnome_config_get_int("gsteak/general/english=1");
	german_variable = gnome_config_get_int("gsteak/general/german=1");

	gtk_widget_realize(applet);
	counter = create_counter();
	applet_widget_add(APPLET_WIDGET(applet), counter);
	gtk_widget_show_all(applet);
	gtk_widget_set_usize(GTK_WIDGET(applet),
	  APPLET_WIDTH,
	  APPLET_HEIGHT);

	gtk_signal_connect(GTK_OBJECT(applet), "delete_event",
			   GTK_SIGNAL_FUNC(bye), NULL);
	gtk_signal_connect(GTK_OBJECT(applet), "save_session",
			   GTK_SIGNAL_FUNC(save_session), NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					      "about",                    
					      GNOME_STOCK_MENU_ABOUT,
					      _("About"),         
					      GTK_SIGNAL_FUNC(about_cb),
					      NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					      "preference",                    
					      GNOME_STOCK_MENU_PREF,
					      _("Preferences"),         
					      GTK_SIGNAL_FUNC(prefs_cb),
					      NULL);

	applet_widget_gtk_main();
	return(0);
}
